
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QDebug>
#include <QtGui/QPainter>
#include <QtGui/QStyleOptionGraphicsItem>
#include <QtGui/QGraphicsScene>

#include "BackgroundItem.h"

const int decoMargin = 2; // default pen width + supplementary contour if selected (so 1 + 1)
const int penWidth = 1;
const int cornerSize = 2;

BackgroundItem::BackgroundItem ( QGraphicsItem * parent ) : QGraphicsItem(parent),
    m_freezeXAxis(false),
    m_freezeYAxis(false)
{
    setFlag(QGraphicsItem::ItemIsMovable, true);
    setFlag(QGraphicsItem::ItemIsSelectable, true);
    setFlag(QGraphicsItem::ItemIsFocusable, true);
    setFlag(QGraphicsItem::ItemSendsScenePositionChanges,true);
}

QRectF BackgroundItem::boundingRect() const
{
    QRectF res;
    res = QRectF(
        -( m_size.width() ) / 2,
        -( m_size.height() ) / 2 - decoMargin,
        m_size.width() + 2 * decoMargin,
        m_size.height() + 2 * decoMargin
        );

    return res;
}

void BackgroundItem::setImage(const QImage &image, const QSize &size, const QSize &desktopSize)
{
    m_image = image;
    m_size = size;
    m_desktopSize = desktopSize;

    QRectF itemBoundinRect = boundingRect();
    qreal ih = itemBoundinRect.height() / 2;
    qreal iw = itemBoundinRect.width() / 2;

    setPos(QPointF(iw + 1,ih + 1) - m_offset);

    update();
}

void BackgroundItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget * /* widget */)
{
    QPen pen(Qt::black);
    pen.setWidth(1);
    pen.setCosmetic(true);

    //draw out box
    pen.setBrush(Qt::black);
    painter->setPen(pen);

    QRectF itemRect = boundingRect().translated(decoMargin - penWidth,decoMargin - penWidth);

    painter->drawRoundedRect(itemRect,cornerSize,cornerSize);

    // draw image
    drawDesktop(painter);

    // draw highlight if needed
    if (option->state & QStyle::State_Selected)
    {
        pen.setBrush(Qt::red);
        painter->setPen(pen);
        painter->drawRoundedRect(itemRect, cornerSize, cornerSize);
    }
}

void BackgroundItem::drawDesktop(QPainter *painter)
{
    if ( m_image.isNull() )
    {
        painter->fillRect (
            -m_size.width() / 2,
            -m_size.height() / 2,
            m_size.width(),
            m_size.height(),
            Qt::white );
    }
    else
    {
        QRectF target(
            -m_size.width() / 2,
            -m_size.height() / 2,
            m_size.width(),
            m_size.height()
            );
        painter->drawImage( target,m_image,m_image.rect() );
    }
}

void BackgroundItem::mousePressEvent1 ( QGraphicsSceneMouseEvent * event )
{
    QGraphicsItem::mousePressEvent(event);
}

void BackgroundItem::mouseMoveEvent1 ( QGraphicsSceneMouseEvent * event )
{
    QGraphicsItem::mouseMoveEvent(event);
}

void BackgroundItem::mouseReleaseEvent1 ( QGraphicsSceneMouseEvent * event )
{
    QGraphicsItem::mouseReleaseEvent(event);
}

void BackgroundItem::setFreezeXAxis(bool val)
{
    m_freezeXAxis = val;
}

void BackgroundItem::setFreezeYAxis(bool val)
{
    m_freezeYAxis = val;
}

QVariant BackgroundItem::itemChange(GraphicsItemChange change, const QVariant &value)
{
    //qDebug() << " BackgroundItem::itemChange = " << change;
    if (change == QGraphicsItem::ItemPositionChange)
    {
        if (m_freezeXAxis)
        {
            return QPointF( pos().x(), value.toPointF().y() );
        }
        else if (m_freezeYAxis)
        {
            return QPointF( value.toPointF().x(), pos().y() );
        }
        else  // limit to the scene rect
        {
            // value is the new position.
            QPointF newPos = value.toPointF();
            if ( scene() )
            {
                QRectF itemBoundinRect = boundingRect();

                qreal ih = itemBoundinRect.height() / 2;
                qreal iw = itemBoundinRect.width() / 2;
                QRectF rect = scene()->sceneRect();

                rect.adjust(iw - 1,ih - 1,-iw + 1,-ih + 1);

                if ( rect.isValid() && !rect.contains(newPos) )
                {
                    // Keep the item inside the scene rect.
                    newPos.setX( qMin( rect.right(), qMax( newPos.x(), rect.left() ) ) );
                    newPos.setY( qMin( rect.bottom(), qMax( newPos.y(), rect.top() ) ) );
                    return newPos;
                }
            }
        }
    }
    return QGraphicsItem::itemChange(change, value);
}

void BackgroundItem::setOffset(const QPointF &offset)
{
    m_offset = offset;
}

QPointF BackgroundItem::getOffset() const
{
    QPointF p = scenePos();
    QRectF itemBoundinRect = boundingRect();
    qreal ih = itemBoundinRect.height() / 2;
    qreal iw = itemBoundinRect.width() / 2;
    return p + QPointF(-iw - 1,-ih - 1);
}
