
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QFileInfo>
#include <QtCore/QDir>
#include <QtCore/QDebug>

#include <QtGui/QDesktopWidget>
#include <QtGui/QColorDialog>

#include "qexifimageheader.h"

#include "BackgroundPainterConfigWidget.h"

#include "ui_BackgroundPainterConfigWidget.h"

#include "ScreenItem.h"
#include "DesktopScene.h"
#include "AppSettings.h"
#include "MiscFunctions.h"
#include "LogHandler.h"
#include "NetworkMessage.h"

BackgroundPainterConfigWidget::BackgroundPainterConfigWidget(QWidget *parent)
    : QWidget(parent),
    ui(new Ui::BackgroundPainterConfigWidget),
    m_exportParameters(true),
    m_ignoreParameters(false),
    m_skipTooSmallImage(false),
    m_rotateFromMetadata(false),
    m_keepWMSettings(false)
{
    ui->setupUi(this);
    createDesktop();
    loadSettings();
    updateUI();
    updateDesktop();
}

BackgroundPainterConfigWidget::~BackgroundPainterConfigWidget()
{
    //saveSettings();
    delete ui;
}

void BackgroundPainterConfigWidget::createDesktop()
{
    QDesktopWidget *desktop = QApplication::desktop();

    qDeleteAll(m_screens);
    m_screens.clear();

    for (int screen = 0; screen < desktop->screenCount(); screen++)
    {
        QRect avail = desktop->screenGeometry(screen);
        m_screens << new ScreenItem(avail.size(),screen + 1);
    }
}

void BackgroundPainterConfigWidget::updateDesktop()
{
    if ( m_screens.isEmpty() )
        return;

    //qDebug() << "scene nb items = " << m_desktopScene->items().size();

    foreach ( QGraphicsItem * item, m_desktopScene->items() )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
            m_desktopScene->removeItem(item);
    }

    QDesktopWidget *desktop = QApplication::desktop();
    int mainScreen = desktop->primaryScreen();

    QList<ScreenItem *> screens;

    if ( m_screens.size() > 1 && m_painter.getShareImageBetweenScreen() && m_painter.getMultiMonitorSupportType() != BackgroundPainter::MULTIMONITOR_NO_MULTIMONITOR )
    {
        for (int screen = 0; screen < m_screens.size(); screen++)
        {
            screens << m_screens.at(screen);
            screens.at(screen)->setUseMargins(true);
            screens.at(screen)->setDrawText(true);
        }
    }
    else
    {
        screens << m_screens.at(mainScreen);
        screens.at(0)->setUseMargins(false); // single screen => no margin
        screens.at(0)->setDrawText(false); // single screen => no text
    }

    m_desktopScene->addScreens(screens);
    ui->graphicsViewDesktop->zoomToFit();
}

void BackgroundPainterConfigWidget::updateUI()
{
    m_desktopScene = new DesktopScene(this);
    m_desktopScene->setSceneState(DesktopScene::STATE_MOVING_SCREEN);
    ui->graphicsViewDesktop->setScene(m_desktopScene);

    connect( m_desktopScene, SIGNAL( reordering() ), this, SLOT( previewImage() ) );

    // setup UI
    initMultiMonitorSupportType();
    initResizingType();
    initAlignmentType();
    initFillingType();
    initSpecialEffectType();
    initSamplingMethod();

    // apply parameters
    applyConfig();

    // connect signals if screens configuration change
    connect( QApplication::desktop(),SIGNAL( screenCountChanged(int) ),this,SLOT( updateScreenConfig(int) ) );
    connect( QApplication::desktop(),SIGNAL( resized(int) ),this,SLOT( updateScreenConfig(int) ) );

    // force first tab (basic parameters)
    ui->tabWidgetParameters->setCurrentIndex(0);
}

void BackgroundPainterConfigWidget::updateScreenConfig(int)
{
    QDesktopWidget *desktop = QApplication::desktop();

    // ? new screen
    if ( desktop->screenCount() > m_screens.size() )
    {
        for (int screen = m_screens.size(); screen < desktop->screenCount(); screen++)
        {
            QRect avail = desktop->screenGeometry(screen);
            m_screens << new ScreenItem(avail.size(),screen + 1);
        }
    }
    else if ( desktop->screenCount() < m_screens.size() )
    {
        int nb = m_screens.size();
        for (int screen = desktop->screenCount(); screen < nb; screen++)
            delete m_screens.takeLast();
    }

    // update geometry and screen  number
    for (int screen = 0; screen < m_screens.size(); screen++)
    {
        QRect avail = desktop->screenGeometry(screen);
        m_screens[screen]->setSize( avail.size() );
        m_screens[screen]->setNumber(screen + 1);
    }

    updateDesktop();

    processImage(m_lastFile);
}

void BackgroundPainterConfigWidget::applyConfig()
{
    // enable only if several screens detected ...
    ui->checkBoxShareScreen->setChecked( m_painter.getShareImageBetweenScreen() );
    ui->checkBoxShareScreen->setEnabled(QApplication::desktop()->screenCount() > 1);

    ui->comboBoxMultiMonitor->setCurrentIndex( m_painter.getMultiMonitorSupportType() );

    // enable only if several screens detected ...
    ui->comboBoxMultiMonitor->setEnabled( QApplication::desktop()->screenCount() > 1 && m_painter.getShareImageBetweenScreen() );

    // display tips only if several screens
    ui->labelTips->setVisible(QApplication::desktop()->screenCount() > 1);

    ui->comboBoxResizingType->setCurrentIndex( m_painter.getResizingType() );
    ui->comboBoxAlignmentTypeVertical->setCurrentIndex( m_painter.getVerticalAlignmentType() );
    ui->comboBoxAlignmentTypeVertical->setEnabled(m_painter.getResizingType() != BackgroundPainter::SCALE_IGNORE_ASPECT &&
                                                  m_painter.getResizingType() != BackgroundPainter::SCALE_KEEP_ASPECT
                                                  );
    ui->comboBoxAlignmentTypeHorizontal->setCurrentIndex( m_painter.getHorizontalAlignmentType() );
    ui->comboBoxAlignmentTypeHorizontal->setEnabled(m_painter.getResizingType() != BackgroundPainter::SCALE_IGNORE_ASPECT &&
                                                    m_painter.getResizingType() != BackgroundPainter::SCALE_KEEP_ASPECT
                                                    );
    ui->comboBoxFillingType->setCurrentIndex( m_painter.getFillingType() );
    ui->comboBoxFillingType->setEnabled(m_painter.getResizingType() != BackgroundPainter::SCALE_IGNORE_ASPECT &&
                                        m_painter.getResizingType() != BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT
                                        );

    setButtonColor(m_painter.getFillingColor1(),ui->pushButtonPreviewColor1);
    setButtonColor(m_painter.getFillingColor2(),ui->pushButtonPreviewColor2);
    ui->pushButtonPreviewColor1->setEnabled(m_painter.getFillingType() == BackgroundPainter::FILL_BY_COLOR ||
                                            m_painter.getFillingType() == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
                                            m_painter.getFillingType() == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
                                            m_painter.getFillingType() == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
                                            );
    ui->pushButtonPreviewColor2->setEnabled(m_painter.getFillingType() == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
                                            m_painter.getFillingType() == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
                                            m_painter.getFillingType() == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
                                            );

    ui->checkBoxInverseGradient->setEnabled(
        m_painter.getFillingType() == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
        m_painter.getFillingType() == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
        m_painter.getFillingType() == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
        );
    ui->checkBoxInverseGradient->setChecked( m_painter.getInverseGradient() );

    ui->comboBoxSpecialEffect->setCurrentIndex( m_painter.getSpecialEffectType() );
    ui->doubleSpinBoxSpecialEffectParam->setValue( m_painter.getSpecialEffectParameter() );
    ui->doubleSpinBoxSpecialEffectParam->setEnabled( m_painter.hasSpecialEffectParameter( m_painter.getSpecialEffectType() ) );

    ui->checkBoxExportParameters->setChecked(m_exportParameters);
    ui->checkBoxIgnoreParameters->setChecked(m_ignoreParameters);
    ui->checkBoxSkipTooSmallImage->setChecked(m_skipTooSmallImage);
    ui->checkBoxSkipTooSmallImage->setChecked(m_rotateFromMetadata);
    ui->checkBoxKeepWMSettings->setChecked(m_keepWMSettings);
#ifndef Q_WS_WIN // currently only available on windows
    ui->checkBoxKeepWMSettings->setEnabled(false);
#endif

    ui->checkBoxSkipTooSmallImage->setEnabled(m_painter.getResizingType() == BackgroundPainter::SCALE_IGNORE_ASPECT ||
                                              m_painter.getResizingType() == BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT
                                              );

    ui->comboBoxSamplingMethod->setCurrentIndex( m_painter.getSmoothAlgorithm() );
}

void BackgroundPainterConfigWidget::initResizingType()
{
    // user data are set just to help (not used)
    ui->comboBoxResizingType->addItem(tr("no resize"),BackgroundPainter::SCALE_NO_RESIZE);
    ui->comboBoxResizingType->addItem(tr("Stretch to fill (Ignore aspect ratio)"),BackgroundPainter::SCALE_IGNORE_ASPECT);
    ui->comboBoxResizingType->addItem(tr("Proportionally stretch to fit"),BackgroundPainter::SCALE_KEEP_ASPECT);
    ui->comboBoxResizingType->addItem(tr("Fill proportionally"),BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT);
}

void BackgroundPainterConfigWidget::initMultiMonitorSupportType()
{
    // user data are set just to help (not used)
    ui->comboBoxMultiMonitor->addItem(tr("No multi monitor support (wallpaper duplicated)"),BackgroundPainter::MULTIMONITOR_NO_MULTIMONITOR);
    ui->comboBoxMultiMonitor->addItem(tr("Extend image (one file for all monitors)"),BackgroundPainter::MULTIMONITOR_ONE_IMAGE_MULTIPLEMONITOR);
    ui->comboBoxMultiMonitor->addItem(tr("Extend image then crop (one file per monitor)"),BackgroundPainter::MULTIMONITOR_ONE_IMAGE_BY_MONITOR);
}

void BackgroundPainterConfigWidget::initAlignmentType()
{
    // user data are set just to help (not used)
    ui->comboBoxAlignmentTypeVertical->addItem(tr("Top alignment"),BackgroundPainter::ALIGN_V_TOP);
    ui->comboBoxAlignmentTypeVertical->addItem(tr("Center alignment"),BackgroundPainter::ALIGN_V_CENTER);
    ui->comboBoxAlignmentTypeVertical->addItem(tr("Bottom alignment"),BackgroundPainter::ALIGN_V_BOTTOM);

    // user data are set just to help (not used)
    ui->comboBoxAlignmentTypeHorizontal->addItem(tr("Left alignment"),BackgroundPainter::ALIGN_H_LEFT);
    ui->comboBoxAlignmentTypeHorizontal->addItem(tr("Center alignment"),BackgroundPainter::ALIGN_H_CENTER);
    ui->comboBoxAlignmentTypeHorizontal->addItem(tr("Right alignment"),BackgroundPainter::ALIGN_H_RIGHT);
}

void BackgroundPainterConfigWidget::initFillingType()
{
    // user data are set just to help (not used)
    ui->comboBoxFillingType->addItem(tr("Fixed color"),BackgroundPainter::FILL_BY_COLOR);
    ui->comboBoxFillingType->addItem(tr("Smart color (extract from image)"),BackgroundPainter::FILL_BY_SMART_COLOR);
    ui->comboBoxFillingType->addItem(tr("Blurred image"),BackgroundPainter::FILL_BY_BLUR);
    ui->comboBoxFillingType->addItem(tr("Vertical gradient"),BackgroundPainter::FILL_BY_VERTICAL_GRADIENT);
    ui->comboBoxFillingType->addItem(tr("Horizontal gradient"),BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT);
    ui->comboBoxFillingType->addItem(tr("Radial gradient"),BackgroundPainter::FILL_BY_RADIAL_GRADIENT);
}

void BackgroundPainterConfigWidget::initSamplingMethod()
{
    // user data are set just to help (not used)
    ui->comboBoxSamplingMethod->addItem(tr("Qt default algorithm"),BackgroundPainter::SMOOTH_DEFAULT);
    ui->comboBoxSamplingMethod->addItem(tr("Blackman smooth filter"),BackgroundPainter::SMOOTH_BLITZ_BLACKMANFILTER);
    ui->comboBoxSamplingMethod->addItem(tr("Cubic filter"),BackgroundPainter::SMOOTH_BLITZ_CUBICFILTER);
    ui->comboBoxSamplingMethod->addItem(tr("Mitchell filter"),BackgroundPainter::SMOOTH_BLITZ_MITCHELLFILTER);
    ui->comboBoxSamplingMethod->addItem(tr("Lanczos filter"),BackgroundPainter::SMOOTH_BLITZ_LANCZOSFILTER);
}

void BackgroundPainterConfigWidget::initSpecialEffectType()
{
    ui->comboBoxSpecialEffect->addItem(tr("No effect"),BackgroundPainter::EFFECT_NONE);
    ui->comboBoxSpecialEffect->addItem(tr("Grayscale"),BackgroundPainter::EFFECT_GRAYSCALE);
    ui->comboBoxSpecialEffect->addItem(tr("Blur"),BackgroundPainter::EFFECT_BLUR);
    ui->comboBoxSpecialEffect->addItem(tr("Gaussian blur"),BackgroundPainter::EFFECT_GAUSSIAN_BLUR);
    ui->comboBoxSpecialEffect->addItem(tr("Edge"),BackgroundPainter::EFFECT_EDGE);
    ui->comboBoxSpecialEffect->addItem(tr("Sharpen"),BackgroundPainter::EFFECT_SHARPEN);
    ui->comboBoxSpecialEffect->addItem(tr("Gaussian sharpen"),BackgroundPainter::EFFECT_GAUSSIAN_SHARPEN);
    ui->comboBoxSpecialEffect->addItem(tr("Emboss"),BackgroundPainter::EFFECT_EMBOSS);
    ui->comboBoxSpecialEffect->addItem(tr("Desaturate"),BackgroundPainter::EFFECT_DESATURATE);
    ui->comboBoxSpecialEffect->addItem(tr("Flatten"),BackgroundPainter::EFFECT_FLATTEN);
    ui->comboBoxSpecialEffect->addItem(tr("Swirl"),BackgroundPainter::EFFECT_SWIRL);
    ui->comboBoxSpecialEffect->addItem(tr("Oilpaint"),BackgroundPainter::EFFECT_OILPAINT);
    ui->comboBoxSpecialEffect->addItem(tr("Charcoal"),BackgroundPainter::EFFECT_CHARCOAL);
    ui->comboBoxSpecialEffect->addItem(tr("Sepia"),BackgroundPainter::EFFECT_SEPIA);
    ui->comboBoxSpecialEffect->addItem(tr("Invert colors"),BackgroundPainter::EFFECT_INVERTCOLOR);
    ui->comboBoxSpecialEffect->addItem(tr("Glow"),BackgroundPainter::EFFECT_GLOW);
    ui->comboBoxSpecialEffect->addItem(tr("Progressive blur"),BackgroundPainter::EFFECT_PROGRESSIVEBLUR);
    ui->comboBoxSpecialEffect->addItem(tr("Random Effect"),BackgroundPainter::EFFECT_RANDOM);
}

void BackgroundPainterConfigWidget::previewImage(const QString &file)
{
    if ( !file.isEmpty() && m_previewFile != file )
    {
        m_previewImage = QImage(file);
        ui->labelPreviewImage->setText( QString("%1 - %2x%3").arg( QFileInfo(file).baseName() ).arg( m_previewImage.width() ).arg( m_previewImage.height() ) );
        if (m_rotateFromMetadata)
        {
            QExifImageHeader qexif(file);
            m_previewImage = m_painter.transform( m_previewImage,qexif.value(QExifImageHeader::Orientation).toShort() );
        }
    }

    m_previewFile = file;

    if ( m_previewImage.isNull() )
    {
        m_previewImage = QImage(":/syncwall/flower.png");
        ui->labelPreviewImage->setText("");
    }

    // refresh final size just in case ...
    m_painter.setTargetSize( m_desktopScene->getSize() );
    m_painter.setTargetSizeWithMargins( m_desktopScene->getSizeWithMargins() );

    QSize minDesktopSize = m_desktopScene->getSizeWithMargins();

    //qDebug() << "minDesktopSize = " << minDesktopSize;

    //QSize totalSizeWithConfig = m_painter.getSizeAccordingToConfig(minDesktopSize);
    QSize resampledSrcImgSize = m_painter.getSizeAccordingToConfig( m_previewImage.size() );

    //qDebug() << "resampledSrcImgSize = " << resampledSrcImgSize;

    QImage background = m_painter.processImage(m_previewImage,true);

    //background.save("d:/tmp/background.png","PNG");

    QImage res = m_painter.crop(background,minDesktopSize,true);

    //res.save("d:/tmp/res.png","PNG");

    m_desktopScene->applyImage(res,background,minDesktopSize, resampledSrcImgSize);
}

void BackgroundPainterConfigWidget::processImage(const QString &file, bool manual, bool force)
{
    m_lastFile = file;

    if (m_previewFile != m_lastFile || force)
    {
        m_previewImage = QImage(file);
        if (m_rotateFromMetadata)
        {
            QExifImageHeader qexif(file);
            m_previewImage = m_painter.transform( m_previewImage,qexif.value(QExifImageHeader::Orientation).toShort() );
        }
        m_previewFile = file;
    }

    static int rejectNum = 0;
    if (manual && m_skipTooSmallImage)
    {
        // check a
        QSize s = m_desktopScene->getSize();
        if (m_previewImage.size().width() < s.width() / 2 || m_previewImage.size().height() < s.height() / 2)
        {
            LogHandler::getInstance()->reportInfo( QString("BackgroundPainterConfigWidget: image %1 is reject because its size is too small (%2x%3/%4x%5))").arg( m_lastFile).arg( m_previewImage.size().width() ).arg( m_previewImage.size().height() ).arg( s.width() ).arg( s.height() ) );
            rejectNum++;

            // watchdog
            if (rejectNum < 50)
                emit requestNewImage();
            else
                LogHandler::getInstance()->reportError( QString("BackgroundPainterConfigWidget: too many reject files => please change settings of add more hi-resolution images!!") );
            return;
        }
    }
    rejectNum = 0; // reset the reject counter

    QImage wallp = m_painter.processImage(m_previewImage);

    //wallp.save("C:\\Users\\xbee\\AppData\\Local\\TheHive\\SyncWall\\ori.png");
    QList<QRectF> listGamingArea = m_desktopScene->getGamingAreaList();

    //qDebug() << "BackgroundPainterConfigWidget::processImage = " << listGamingArea;

    if (m_painter.getMultiMonitorSupportType() != BackgroundPainter::MULTIMONITOR_ONE_IMAGE_BY_MONITOR)
    {
        QImage res = m_painter.cropAndMerge(wallp,listGamingArea);
        emit newImage(res,m_keepWMSettings);
    }
    else
    {
        QList<QImage> res = m_painter.cropAndSplit(wallp,listGamingArea);
        emit newImages(res,m_keepWMSettings);
    }

    previewImage(file); // load m_previewImage

    // encode configuration and send it to client
    emit newConfiguration( encodeConfiguration(file) );

    NetworkMessage mess;
    mess.options = encodeConfiguration("");
    mess.image = QImage(file);
    emit newConfiguration(mess);
}

void BackgroundPainterConfigWidget::on_checkBoxShareScreen_stateChanged ( int state )
{
    m_painter.setShareImageBetweenScreen(state);

    ui->checkBoxShareScreen->setChecked( m_painter.getShareImageBetweenScreen() );

    // enable only if several screens detected ...
    ui->comboBoxMultiMonitor->setEnabled( QApplication::desktop()->screenCount() > 1 && m_painter.getShareImageBetweenScreen() );

    // enable tips label if needed
    ui->labelTips->setEnabled(state == Qt::Checked);

    emit parametersChange();

    updateDesktop();
}

void BackgroundPainterConfigWidget::on_checkBoxInverseGradient_stateChanged ( int state )
{
    if ( (bool)state == m_painter.getInverseGradient() )
        return;

    m_painter.setInverseGradient(state);
    previewImage();
}

void BackgroundPainterConfigWidget::on_checkBoxIgnoreParameters_stateChanged ( int state )
{
    m_ignoreParameters = state;
}

void BackgroundPainterConfigWidget::on_checkBoxExportParameters_stateChanged ( int state )
{
    m_exportParameters = state;
}

void BackgroundPainterConfigWidget::on_checkBoxKeepWMSettings_stateChanged ( int state )
{
    m_keepWMSettings = state;
}

void BackgroundPainterConfigWidget::on_checkBoxSkipTooSmallImage_stateChanged ( int state )
{
    m_skipTooSmallImage = state;
}

void BackgroundPainterConfigWidget::on_checkBoxRotateWithMetadata_stateChanged ( int state )
{
    m_rotateFromMetadata = state;
}

void BackgroundPainterConfigWidget::on_comboBoxResizingType_activated(int index)
{
    if ( index == m_painter.getResizingType() )
        return;

    m_painter.setResizingType(index);
    ui->comboBoxAlignmentTypeVertical->setEnabled(
        index != BackgroundPainter::SCALE_IGNORE_ASPECT &&
        index != BackgroundPainter::SCALE_KEEP_ASPECT
        );
    ui->comboBoxAlignmentTypeHorizontal->setEnabled(
        index != BackgroundPainter::SCALE_IGNORE_ASPECT &&
        index != BackgroundPainter::SCALE_KEEP_ASPECT
        );

    ui->comboBoxFillingType->setEnabled(
        index != BackgroundPainter::SCALE_IGNORE_ASPECT &&
        index != BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT
        );

    ui->checkBoxSkipTooSmallImage->setEnabled(index == BackgroundPainter::SCALE_IGNORE_ASPECT ||
                                              index == BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT
                                              );

    emit parametersChange();

    previewImage();
}

void BackgroundPainterConfigWidget::on_comboBoxMultiMonitor_activated(int index)
{
    if ( index == m_painter.getMultiMonitorSupportType() )
        return;

    m_painter.setMultiMonitorSupportType(index);

    emit parametersChange();

    updateDesktop();
}

void BackgroundPainterConfigWidget::on_comboBoxAlignmentTypeVertical_activated(int index)
{
    if ( index == m_painter.getVerticalAlignmentType() )
        return;

    m_painter.setVerticalAlignmentType(index);

    emit parametersChange();

    previewImage();
}

void BackgroundPainterConfigWidget::on_comboBoxAlignmentTypeHorizontal_activated(int index)
{
    if ( index == m_painter.getHorizontalAlignmentType() )
        return;

    m_painter.setHorizontalAlignmentType(index);

    emit parametersChange();

    previewImage();
}

void BackgroundPainterConfigWidget::on_comboBoxSpecialEffect_activated(int index)
{
    if ( index == m_painter.getSpecialEffectType() )
        return;

    m_painter.setSpecialEffectType(index);
    ui->doubleSpinBoxSpecialEffectParam->setEnabled( m_painter.hasSpecialEffectParameter(index) );
    previewImage();
}

void BackgroundPainterConfigWidget::on_doubleSpinBoxSpecialEffectParam_valueChanged(double val)
{
    if ( val == m_painter.getSpecialEffectParameter() )
        return;

    m_painter.setSpecialEffectParameter(val);
    previewImage();
}

void BackgroundPainterConfigWidget::on_comboBoxSamplingMethod_activated(int index)
{
    if ( index == m_painter.getSmoothAlgorithm() )
        return;

    m_painter.setSmoothAlgorithm(index);

    previewImage();
}

void BackgroundPainterConfigWidget::on_comboBoxFillingType_activated(int index)
{
    if ( index == m_painter.getFillingType() )
        return;

    m_painter.setFillingType(index);
    ui->pushButtonPreviewColor1->setEnabled(index == BackgroundPainter::FILL_BY_COLOR ||
                                            index == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
                                            );
    ui->pushButtonPreviewColor2->setEnabled(index == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
                                            );
    ui->checkBoxInverseGradient->setEnabled(index == BackgroundPainter::FILL_BY_VERTICAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_HORIZONTAL_GRADIENT ||
                                            index == BackgroundPainter::FILL_BY_RADIAL_GRADIENT
                                            );
    previewImage();
}

void BackgroundPainterConfigWidget::on_pushButtonPreviewColor1_pressed()
{
    QColor prevColor = ui->pushButtonPreviewColor1->palette().color(QPalette::Button);
    QColor c = QColorDialog::getColor(prevColor);

    if (c == prevColor)
        return;

    setButtonColor(c,ui->pushButtonPreviewColor1);
    m_painter.setFillingColor1(c);
    previewImage();
}

void BackgroundPainterConfigWidget::on_pushButtonPreviewColor2_pressed()
{
    QColor prevColor = ui->pushButtonPreviewColor2->palette().color(QPalette::Button);
    QColor c = QColorDialog::getColor(prevColor);

    if (c == prevColor)
        return;

    setButtonColor(c,ui->pushButtonPreviewColor2);
    m_painter.setFillingColor2(c);
    previewImage();
}

void BackgroundPainterConfigWidget::setButtonColor(const QColor &c, QPushButton *b)
{
    if ( c.isValid() && b)
    {
        QPalette palette = b->palette();
        palette.setColor(QPalette::Button, c);
        b->setPalette(palette); // to easily save the color
    }
}

void BackgroundPainterConfigWidget::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("ScreensConfig");

    // screens
    int size = settings.beginReadArray("screens");

    for (int i = 0; i < size; ++i)
    {
        settings.setArrayIndex(i);
        int num = settings.value("num").toInt();
        int marginTop = settings.value("marginTop").toInt();
        int marginBottom = settings.value("marginBottom").toInt();
        int marginLeft = settings.value("marginLeft").toInt();
        int marginRight = settings.value("marginRight").toInt();
        QPointF pos = settings.value("pos").toPointF();

        if ( num <= m_screens.size() && num > 0 )
        {
            m_screens.at(num - 1)->setMargings(marginTop,marginBottom,marginLeft,marginRight);
            m_screens.at(num - 1)->setPos(pos); // to restore good placement
        }
    }

    settings.endArray();

    settings.endGroup();

    settings.beginGroup("ClientServerConnection");
    m_exportParameters = settings.value("exportParameters",true).toBool();
    m_ignoreParameters = settings.value("ignoreParameters",false).toBool();
    settings.endGroup();

    settings.beginGroup("BackgroundPainter");
    m_skipTooSmallImage = settings.value("skipTooSmallImage",false).toBool();
    m_rotateFromMetadata = settings.value("rotateFromMetadata",false).toBool();
    m_keepWMSettings = settings.value("keepWMSettings",false).toBool();

    settings.endGroup();
}

void BackgroundPainterConfigWidget::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("ScreensConfig");

    // screens
    settings.beginWriteArray( "screens", m_screens.count() );

    for (int i = 0; i < m_screens.size(); ++i)
    {
        settings.setArrayIndex(i);
        settings.setValue( "num", m_screens.at(i)->getNumber() );
        settings.setValue( "marginTop", m_screens.at(i)->getMarginTop() );
        settings.setValue( "marginBottom", m_screens.at(i)->getMarginBottom() );
        settings.setValue( "marginLeft", m_screens.at(i)->getMarginLeft() );
        settings.setValue( "marginRight", m_screens.at(i)->getMarginRight() );
        settings.setValue( "pos", m_screens.at(i)->pos() );
    }

    settings.endArray();

    settings.endGroup();

    settings.beginGroup("ClientServerConnection");

    settings.setValue("exportParameters",m_exportParameters);
    settings.setValue("ignoreParameters",m_ignoreParameters);

    settings.endGroup();

    settings.beginGroup("BackgroundPainter");
    settings.setValue("skipTooSmallImage",m_skipTooSmallImage);
    settings.setValue("rotateFromMetadata",m_rotateFromMetadata);
    settings.setValue("keepWMSettings",m_keepWMSettings);

    //settings.setValue("lastFile",m_lastFile);

    settings.endGroup();

    m_painter.saveSettings();
}

void BackgroundPainterConfigWidget::decodeAndApplyMessage(const NetworkMessage &mess)
{
    NetworkMessage decoded;
    decoded.fromByteArray(mess.data);
    decodeAndApplyMessage(decoded.options);

    // save network image

    if ( !decoded.image.isNull() )
    {
        QString tmpFile = MiscFunctions::getValidStorage() + QDir::separator () + "network.png";
        LogHandler::getInstance()->reportDebug( QString("BackgroundPainterConfigWidget: Save network image in %1").arg( tmpFile ) );
        if( !decoded.image.save(tmpFile,"PNG") )
            LogHandler::getInstance()->reportDebug( QString("BackgroundPainterConfigWidget: Can't save network image in %1").arg( tmpFile ) );

        emit newNetworkImage(tmpFile);
    }
}

void BackgroundPainterConfigWidget::decodeAndApplyMessage(const QString &mess)
{
    QStringList list = mess.split('|');
    QString version = list[0];
    if (version == "1.1")
    {
        bool hasParameters = (list.size() == 12) ? true : false;
        bool hasJustHeader = (list.size() == 2) ? true : false;
        if (!hasParameters && !hasJustHeader)
        {
            LogHandler::getInstance()->reportError( QString("BackgroundPainterConfigWidget: Message with invalid nb of parameters (%1 instead of %2)").arg( list.size() ).arg( 10 ) );
            return;
        }

        int index = 1;
        QString file = list[index++];
        bool somethingDiffer = ( file != MiscFunctions::getSignificantPart(m_lastFile) ); // check if new request is different from previous
        if (hasParameters && !m_ignoreParameters)
        {
            // m_painter.setResizingType()
            somethingDiffer |= ( m_painter.getResizingType() != list[index].toInt() );
            m_painter.setResizingType( list[index++].toInt() );

            // m_painter.setVerticalAlignmentType()
            somethingDiffer |= ( m_painter.getVerticalAlignmentType() != list[index].toInt() );
            m_painter.setVerticalAlignmentType( list[index++].toInt() );

            // m_painter.setHorizontalAlignmentType()
            somethingDiffer |= ( m_painter.getHorizontalAlignmentType() != list[index].toInt() );
            m_painter.setHorizontalAlignmentType( list[index++].toInt() );

            // m_painter.setFillingType()
            somethingDiffer |= ( m_painter.getFillingType() != list[index].toInt() );
            m_painter.setFillingType( list[index++].toInt() );

            // m_painter.getFillingColor1
            somethingDiffer |= ( m_painter.getFillingColor1() != QColor(list[index]) );
            m_painter.setFillingColor1( QColor(list[index++]) );

            // m_painter.getFillingColor2
            somethingDiffer |= ( m_painter.getFillingColor2() != QColor(list[index]) );
            m_painter.setFillingColor2( QColor(list[index++]) );

            // m_painter.setInverseGradient
            somethingDiffer |= ( m_painter.getInverseGradient() != (bool)list[index].toInt() );
            m_painter.setInverseGradient( (bool)list[index++].toInt() );

            // m_painter.setSpecialEffectType()
            somethingDiffer |= ( m_painter.getSpecialEffectType() != list[index].toInt() );
            m_painter.setSpecialEffectType( list[index++].toInt() );

            // m_painter.setSpecialEffectParameter();
            somethingDiffer |= ( m_painter.getSpecialEffectParameter() != list[index].toFloat() );
            m_painter.setSpecialEffectParameter( list[index++].toFloat() );

            // m_painter.setSmoothAlgorithm();
            somethingDiffer |= ( m_painter.getSmoothAlgorithm() != list[index].toInt() );
            m_painter.setSmoothAlgorithm( list[index++].toInt() );
        }

        if (somethingDiffer)
        {
            applyConfig();
            if ( !file.isEmpty() )
                emit decodedImage(file);
        }
    }
    else
    {
        LogHandler::getInstance()->reportError( QObject::tr("BackgroundPainterConfigWidget: Message with unknown version received (%1)").arg( version ) );
    }
}

QString BackgroundPainterConfigWidget::encodeConfiguration(const QString &file)
{
    // version string
    QString mess = "1.1";

    // file
    mess += "|" + MiscFunctions::getSignificantPart(file);

    // export image parameters to client ?
    if (!m_exportParameters)
        return mess;

    // m_painter.getResizingType()
    mess += QString("|%1").arg( m_painter.getResizingType() );

    // m_painter.getVerticalType()
    mess += QString("|%1").arg( m_painter.getVerticalAlignmentType() );

    // m_painter.getHorizontalAlignmentType()
    mess += QString("|%1").arg( m_painter.getHorizontalAlignmentType() );

    // m_painter.getFillingType()
    mess += QString("|%1").arg( m_painter.getFillingType() );

    // m_painter.getFillingColor1
    mess += QString("|%1").arg( m_painter.getFillingColor1().name() );

    // m_painter.getFillingColor1
    mess += QString("|%1").arg( m_painter.getFillingColor2().name() );

    // m_painter.getInverseGradient
    mess += QString("|%1").arg( m_painter.getInverseGradient() );

    // m_painter.getSpecialEffectType()
    // special RANDOM_EFFECT
    if (m_painter.getSpecialEffectType() != BackgroundPainter::EFFECT_RANDOM)
        mess += QString("|%1").arg( m_painter.getSpecialEffectType() );
    else // send computed effect
        mess += QString("|%1").arg( m_painter.getLastSpecialEffectType() );

    // m_painter.getSpecialEffectParameter();
    mess += QString("|%1").arg( m_painter.getSpecialEffectParameter() );

    // m_painter.getSmoothAlgorithm();
    mess += QString("|%1").arg( m_painter.getSmoothAlgorithm() );

    return mess;
}

bool BackgroundPainterConfigWidget::isUserMoveAllowed() const
{
    return m_painter.getResizingType() == BackgroundPainter::SCALE_KEEP_EXPANDED_ASPECT &&  m_painter.getShareImageBetweenScreen();
}

void BackgroundPainterConfigWidget::setUserOffset(const QPoint &offset)
{
    m_desktopScene->setUserOffset(offset);
    m_painter.setUserOffset(offset);
}

QPointF BackgroundPainterConfigWidget::validCurrentUserPositioning()
{
    QPointF offset( -m_desktopScene->getUserOffset() );
    m_painter.setUserOffset(offset);

    previewImage();
    return offset;
}

int BackgroundPainterConfigWidget::getScreenNumberByConfig() const
{
    if ( m_painter.getShareImageBetweenScreen() && (m_painter.getMultiMonitorSupportType() != BackgroundPainter::MULTIMONITOR_NO_MULTIMONITOR) )
        return QApplication::desktop()->screenCount();
    else
        return 1;
}
