
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QDebug>
#include <QtCore/QFileInfo>
#include <QtCore/QDateTime>
#include <QtCore/QBuffer>

#include <QtSql/QSqlQuery>
#include <QtSql/QSqlError>

#include "DatabaseManager.h"

#include "AppSettings.h"
#include "LogHandler.h"
#include "MiscFunctions.h"
#include "ImageInfos.h"

#if !defined(WIN32)
#include <unistd.h> // for unlink
#endif

DatabaseManager::DatabaseManager(QObject *parent) :
    QObject(parent),
    m_valid(false),
    m_driverName("")
{
    init();
}

DatabaseManager::~DatabaseManager()
{
    m_valid = false;
    m_db.close();
}

bool DatabaseManager::isValid() const
{
    return m_valid;
}

void DatabaseManager::init()
{
    QStringList neededDrivers(QStringList() << "QSQLITE" << "QSQLITE3");

    foreach (const QString &driverName, neededDrivers)
    {
        if ( QSqlDatabase::isDriverAvailable(driverName) )
        {
            m_driverName = driverName;
            break;
        }
    }

    if ( m_driverName.isEmpty() )
    {
        LogHandler::getInstance()->reportError( QString("No SQLITE driver available") );
        return;
    }

    m_database = MiscFunctions::getValidStorage() + QString("/%1.sqlite").arg(PACKAGE_NAME).toLower();

    LogHandler::getInstance()->reportInfo( QString("DatabaseManager::init: open db %1 ...").arg(m_database) );
    if ( !QFileInfo(m_database).exists() && !createDB() )
        return;

    m_valid = openDB();
}

bool DatabaseManager::initDB()
{
    m_db = QSqlDatabase::addDatabase(m_driverName);
    m_db.setDatabaseName(m_database);
    if ( !m_db.open() )
    {
        LogHandler::getInstance()->reportError( QString("DatabaseManager::createDB: db connection failed ... (%1)").arg( m_db.lastError().text() ) );
        return false;
    }

    QSqlQuery query(m_db);

    // init db
    query.exec("PRAGMA journal_mode = OFF");

    return true;
}

void DatabaseManager::clearDB()
{
    if (m_valid)
        m_db.close();
    unlink( m_database.toStdString().c_str() );
    init();
}

bool DatabaseManager::createDB()
{
    if ( !initDB() )
        return false;

    QSqlQuery query(m_db);

    // set default version
    query.exec( QString("pragma user_version = %1").arg(PACKAGE_DB_VERSION) );

    QString request = QString("create table ImageInfos (") +
                      "  url text primary key, " +
                      "  size integer, " +
                      "  width integer, " +
                      "  height integer, " +
                      "  rate integer, " +
                      "  offsetmap text, " +
                      "  lastModification text, " +
                      "  timestamp integer, " +
                      "  thumb blob default null)";

    // create table
    if ( !query.exec(request) )
    {
        m_db.close();
        LogHandler::getInstance()->reportError( QString("DatabaseManager::createDB: cannot create table ... (%1)").arg( query.lastError().text() ) );
        return false;
    }
    m_db.close();
    return true;
}

bool DatabaseManager::openDB()
{
    if ( !initDB() )
        return false;

    QSqlQuery query(m_db);

    if ( query.exec("pragma user_version") )
    {
        query.next();

        int user_version = query.value(0).toInt();

        if ( user_version != QString("%1").arg(PACKAGE_DB_VERSION).toInt() )
        {
            m_db.close();
            LogHandler::getInstance()->reportError( QString("DatabaseManager::openDB: database version don't match current one!") );
            return false;
        }
    }
    return true;
}

void DatabaseManager::retrieveInfos(ImageInfos *infos)
{
    if (!infos || !m_valid)
        return;

    QSqlQuery query;

    if ( query.exec( QString("select url, size, width, height, rate, offsetmap, lastModification, thumb from ImageInfos where url = '%1'").arg( infos->getFilename() ) ) )
    {
        if ( query.next() )
        {
            QString file = query.value(0).toString();
            int size = query.value(1).toInt();
            int width = query.value(2).toInt();
            int height = query.value(3).toInt();
            int rate = query.value(4).toInt();
            QString offsetmap = query.value(5).toString();
            QString lastModification = query.value(6).toString();
            QByteArray ba = query.value(7).toByteArray();

            QImage img;
            img.loadFromData(ba);

            infos->setHeight(height);
            infos->setWidth(width);
            infos->setRate(rate);
            infos->setOffsetMap(offsetmap);
            infos->setLastModificationTime(lastModification);
            infos->setFileSize(size);
            infos->setThumbnail(img);
        }
    }
}

void DatabaseManager::removeInfos(const QString &url)
{
    if (!m_valid)
        return;

    QSqlQuery query;

    if ( !query.exec( QString("DELETE FROM ImageInfos WHERE  url = '%1'").arg(url) ) )
    {
        LogHandler::getInstance()->reportError( QString("DatabaseManager::removeInfos: Error when removing data for %2 (%1) ").arg(url).arg( query.lastError().text() ) );
        return;
    }
}

void DatabaseManager::storeInfos(ImageInfos *infos)
{
    if (!infos || !m_valid)
        return;

    QSqlQuery query;

    /*
       "  url text primary key, " +
        "  size integer, " +
        "  width integer, " +
        "  height integer, " +
        "  lastModification text, "
        "  timestamp integer, " +
     */

    int now = QDateTime::currentDateTime().toTime_t();

    if ( !query.prepare( QString("REPLACE INTO ImageInfos (url, size, width, height, rate, offsetmap, lastModification, timestamp, thumb ) VALUES (:url, :size, :width, :height, :rate, :offsetmap, :lastModification, :timestamp, :thumb)") ) )
    {
        LogHandler::getInstance()->reportError( QString("DatabaseManager::storeInfos: Error in query preparation ... (%1) ").arg( query.lastError().text() ) );
        return;
    }

    query.bindValue( ":url", infos->getFilename() );
    query.bindValue( ":size", infos->getFileSize() );
    query.bindValue( ":width", infos->getWidth() );
    query.bindValue( ":height", infos->getHeight() );
    query.bindValue( ":rate", infos->getRate() );
    query.bindValue( ":offsetmap", infos->getOffsetMap() );
    query.bindValue( ":lastModification", infos->getLastModificationAsString() );
    query.bindValue(":timestamp", now);

    QImage thumb( infos->getThumbnail() );
    QByteArray bytes;
    QBuffer buffer(&bytes);
    buffer.open(QIODevice::WriteOnly);
    thumb.save(&buffer, "PNG");

    query.bindValue(":thumb", bytes);

    if ( !query.exec() )
        LogHandler::getInstance()->reportError( QString("DatabaseManager::storeInfos: Error in query execution ... (%1)").arg( query.lastError().text() ) );
}
