
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtGui/QImage>
#include <QtCore/QFileInfo>
#include <QtCore/QRegExp>

#include <QtGui/QToolTip>
#include <QtGui/QApplication>

#include "ImageInfos.h"
#include "LogHandler.h"

const int thumbnailSize = 100;
const int ImageInfos::maxRating = 5;

ImageInfos::ImageInfos(const QString &file) :
    m_filename(file)
{
    reset();
}

ImageInfos::~ImageInfos()
{
}

void ImageInfos::reset()
{
    m_changed = true;
    m_key = m_filename;
    QFileInfo infos(m_filename);
    m_name = infos.baseName();
    m_completeName = infos.completeBaseName();
    m_fileSize = 0;
    m_width = 0;
    m_height = 0;
    m_rate = 1;
    m_mapOffset.clear();
    m_lastModification = QDateTime::fromString("0-0-0");
    m_thumbnail = computeSubImage();
}

bool ImageInfos::update()
{
    QFileInfo infos(m_filename);

    // file don't change so exit (must do approximative compare for date rounding effect  !!)
    if ( infos.lastModified() <= m_lastModification.addSecs(1) )
        return false;

    m_lastModification = infos.lastModified();
    m_thumbnail = computeSubImage( QImage(m_filename) );
    computeLabel();

    m_fileSize = infos.size();
    m_txtFileSize = sizeToString(m_fileSize);

    return true;
}

QString ImageInfos::sizeToString(quint64 size) const
{
    QString bytesStr;
    if ( !(size >> 10) )
        bytesStr = QString::number(size) + ' ' + QObject::tr("Bytes");
    else if ( !(size >> 20) )
        bytesStr = QString::number(size / 1024.0, 'f', 3) + ' ' + QObject::tr("KB");
    else if ( !(size >> 30) )
        bytesStr = QString::number( (size >> 10) / 1024.0, 'f', 3 ) + ' ' + QObject::tr("MB");
    else if ( !(size >> 40) )
        bytesStr = QString::number( (size >> 20) / 1024.0, 'f', 3 ) + ' ' + QObject::tr("GB");
    else
        bytesStr = QString::number( (size >> 30) / 1024.0, 'f', 3 ) + ' ' + QObject::tr("TB");

    return bytesStr;
}

QImage ImageInfos::computeSubImage(const QImage &image)
{
    // compute thumbnail if needed
    QImage img(image);
    if ( img.isNull() )
    {
        img = QImage(":/syncwall/default.png");
    }
    else
    {
        m_width = img.width();
        m_height = img.height();

        // compute the file size if available
        QString valx = QString::number( img.width() );
        QString valy = QString::number( img.height() );
        m_imageSize = QString("%1x%2").arg(valx).arg(valy);
    }

    return img.scaled(thumbnailSize, thumbnailSize, Qt::KeepAspectRatio);
}

void ImageInfos::setLastModificationTime(const QString &t)
{
    m_lastModification = QDateTime::fromString(t);
}

void ImageInfos::setHeight(int h)
{
    m_height = h;
    m_imageSize = QString("%1x%2").arg(m_width).arg(m_height);
    computeLabel();
}

void ImageInfos::setWidth(int w)
{
    m_width = w;
    m_imageSize = QString("%1x%2").arg(m_width).arg(m_height);
    computeLabel();
}

void ImageInfos::setFileSize(int s)
{
    m_fileSize = s;
    m_txtFileSize = sizeToString(m_fileSize);
}

void ImageInfos::setThumbnail(const QImage &img)
{
    m_thumbnail = img;
}

void ImageInfos::computeLabel()
{
    QString elidedName = QApplication::fontMetrics().elidedText( m_completeName, Qt::ElideLeft, thumbnailSize - 20);
    m_label = QString("%1\n(%2)").arg(elidedName).arg(m_imageSize);
    if (m_rate > 1)
    {
        m_label += " ";
        for (int i = 0; i < m_rate; i++)
            m_label += "*";
    }
}

void ImageInfos::setRate(int rate)
{
    if (rate < 1)
        m_rate = 1;
    else if (rate > maxRating)
        m_rate = maxRating;
    else
        m_rate = rate;
    computeLabel();
}

QString ImageInfos::getOffsetMap() const
{
    QString str;
    QList<int> screens = m_mapOffset.keys();

    for (int i = 0; i < screens.size(); ++i)
        str += QString("%1;%2,%3|").arg(screens[i]).arg( m_mapOffset[screens[i]].x() ).arg( m_mapOffset[screens[i]].y() );

    return str;
}

void ImageInfos::setOffsetMap(const QString &map)
{
    m_mapOffset.clear();

    if ( map.isEmpty() )
        return;

    QRegExp rx("(\\d+);(\\d+),(\\d+)|");

    int pos = 0;
    while ( ( pos = rx.indexIn(map, pos) ) != -1 )
    {
        int nbScr = rx.cap(1).toInt();
        int offx = rx.cap(2).toInt();
        int offy = rx.cap(3).toInt();
        m_mapOffset[nbScr] = QPoint(offx,offy);
        pos += rx.matchedLength() + 1;
    }
}

bool ImageInfos::isOffsetDefined(int nbScr) const
{
    return m_mapOffset.contains(nbScr);
}

QString ImageInfos::getOffsetAsString(int nbScr) const
{
    QString res;
    if ( m_mapOffset.contains(nbScr) )
        res = QString("%1;%2").arg( m_mapOffset[nbScr].x() ).arg( m_mapOffset[nbScr].y() );
    return res;
}

QPoint ImageInfos::getOffset(int nbScr) const
{
    return m_mapOffset[nbScr];
}

void ImageInfos::setOffset(const QPoint &p, int nbScr)
{
    if (nbScr > 0)
        m_mapOffset[nbScr] = p;
}
