
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2012  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *******************************************************************************/

#include "TcpServer.h"
#include "LogHandler.h"

#include <QtNetwork/QHostAddress>
#include <QtNetwork/QTcpSocket>
#include <QtCore/QtEndian>

#include "NetworkMessage.h"

const int defaultPort = 45454;

//---------------------------------------------------------------------------------------------
TcpServer::TcpServer(QObject *parent)
    : QObject(parent),
    m_server( new QTcpServer(parent) ),
    m_currentState(0),
    m_port(defaultPort)
{
    qRegisterMetaType<NetworkMessage>("NetworkMessage");
    connect( m_server, SIGNAL( newConnection() ),
             this, SLOT( receiveNewConnection() ) );

    m_currentState = 0;
    setState(INACTIVE);
}

//---------------------------------------------------------------------------------------------
TcpServer::~TcpServer()
{
    //qDeleteAll(m_clients);
    m_server->close();
    delete m_server;
}

void TcpServer::sendNetworkMessage(const NetworkMessage &mess)
{
    m_currentData = mess.toByteArray();

    //QByteArray block(mess.data);
    //qint64 block_size = qToBigEndian<qint64>(m_currentState.size());

    foreach (QTcpSocket * socket, m_clients)
    {
        sendCurrentData(socket);
    }
}

void TcpServer::sendCurrentData(QTcpSocket *socket)
{
    qint64 block_size = qToBigEndian<qint64>( m_currentData.size() );
    qDebug() << "send block size = " << m_currentData.size();

    // write size
    socket->write( (const char*)&block_size,sizeof(qint64) );

    // write data
    socket->write(m_currentData);
    socket->flush();
}

quint8 TcpServer::getState() const
{
    return m_currentState;
}

//---------------------------------------------------------------------------------------------
QString TcpServer::getStateAsString(quint8 state)
{
    QString state_string = "";

    switch(state)
    {
        case INACTIVE:
            state_string = tr("Inactive");
            break;

        case LISTENING:
            state_string = tr("Listening");
            break;

        case CONNECTED:
            state_string = tr("Connected");
            break;

        default:
            state_string = tr("Unknown");
    }

    return state_string;
}

//---------------------------------------------------------------------------------------------
bool TcpServer::start()
{
    while( !m_server->isListening() && !m_server->listen(QHostAddress::Any, m_port) )
    {
        LogHandler::getInstance()->reportInfo( QString("TcpServer: Unable to start the server: %1").arg( m_server->errorString() ) );
        return false;
    }

    LogHandler::getInstance()->reportInfo( QString("TcpServer: Starting server...") );

    setState(LISTENING);
    return true;
}

//---------------------------------------------------------------------------------------------
void TcpServer::stop()
{
    if( m_server->isListening() )
        m_server->close();

    foreach (QTcpSocket * socket, m_clients)
    socket->abort();

    setState(INACTIVE);
    LogHandler::getInstance()->reportInfo( QString("TcpServer: TcpServer stopped") );
}

//---------------------------------------------------------------------------------------------
void TcpServer::abort()
{
    stop();
}

//---------------------------------------------------------------------------------------------
QString TcpServer::errorString() const
{
    return m_server->errorString();
}

//---------------------------------------------------------------------------------------------
void TcpServer::setState(quint8 new_state)
{
    m_currentState = new_state;
    LogHandler::getInstance()->reportInfo( QString("TcpServer: State changed to '%1'").arg( getStateAsString(new_state) ) );

    emit stateChanged(new_state);
}

//---------------------------------------------------------------------------------------------
void TcpServer::receiveNewConnection()
{
    QTcpSocket *socket = m_server->nextPendingConnection();
    m_clients << socket;
    m_blockSizes.insert(socket,0);
    LogHandler::getInstance()->reportInfo( QString("TcpServer: %1 is connecting...").arg( socket->peerAddress().toString() ) );

    QObject::connect( socket, SIGNAL( readyRead() ),
                      this, SLOT( readMessage() ) );

    QObject::connect( socket, SIGNAL( disconnected() ),
                      this, SLOT( onSocketDisconnected() ) );

    setState(CONNECTED);

    // send the current state to client
    sendCurrentData(socket);
}

//---------------------------------------------------------------------------------------------
void TcpServer::onSocketDisconnected()
{
    QTcpSocket *socket = qobject_cast<QTcpSocket *>( sender() );
    if (!socket)
        return;

    m_clients.removeAll(socket);
    m_blockSizes.remove(socket);

    LogHandler::getInstance()->reportInfo( QString("TcpServer: Connection closed from %1...").arg( socket->peerAddress().toString() ) );

    if ( m_clients.isEmpty() )
        setState(LISTENING);

    emit stateChanged(m_currentState);
}

//---------------------------------------------------------------------------------------------
void TcpServer::readMessage()
{
    QTcpSocket *socket = qobject_cast<QTcpSocket *>( sender() );
    if (!socket)
        return;

    // new block to read
    if(m_blockSizes[socket] == 0)
    {
        if( socket->bytesAvailable() < (int)sizeof(qint64) )
            return;

        QByteArray buffer = socket->read( sizeof(qint64) );
        m_blockSizes[socket] = qFromBigEndian<qint64>( (uchar*)buffer.constData() );
    }

    if(socket->bytesAvailable() <  m_blockSizes[socket])
        return;

    m_buffer = socket->read(m_blockSizes[socket]);

    emit newNetworkMessage( NetworkMessage(m_buffer), socket->localAddress().toString() );
    m_blockSizes[socket] = 0;
}
